<?php

/**
 * Cloudrexx
 *
 * @link      http://www.cloudrexx.com
 * @copyright Cloudrexx AG 2007-2015
 *
 * According to our dual licensing model, this program can be used either
 * under the terms of the GNU Affero General Public License, version 3,
 * or under a proprietary license.
 *
 * The texts of the GNU Affero General Public License with an additional
 * permission and of our proprietary license can be found at and
 * in the LICENSE file you have received along with this program.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 *
 * "Cloudrexx" is a registered trademark of Cloudrexx AG.
 * The licensing of the program under the AGPLv3 does not imply a
 * trademark license. Therefore any rights, title and interest in
 * our trademarks remain entirely with us.
 */

class InstallerCx {
    const FOLDER_NAME_IMAGES = '/images';
    const FOLDER_NAME_MEDIA = '/media';
    const FOLDER_NAME_THEMES = '/themes';

    /**
     * The folder name of the storage location of the core components (/core).
     * Formerly known as ASCMS_CORE_FOLDER.
     * @var string
     */
    const FOLDER_NAME_CORE = '/core';

    /**
     * The folder name of the storage location of the config files (/config).
     * @var string
     */
    const FOLDER_NAME_CONFIG = '/config';

    /**
     * The folder name used for the lib storage location (/lib).
     * Formerly known as ASCMS_LIBRARY_FOLDER
     * @var string
     */
    const FOLDER_NAME_LIBRARY = '/lib';

    /**
     * The webserver's DocumentRoot path.
     * Formerly known as ASCMS_PATH.
     * @var string
     * @access protected
     */
    protected $basePath = null;

    /**
     * The absolute path to the Code Base of the Contrexx installation.
     * Formerly known as ASCMS_DOCUMENT_ROOT.
     * @var string
     * @access protected
     */
    protected $codeBaseDocumentRootPath = null;

    /**
     * The absolute path to the data repository of the Contrexx installation.
     * Formerly known as ASCMS_INSTANCE_DOCUMENT_ROOT.
     * @var string
     * @access protected
     */
    protected $websiteDocumentRootPath = null;

    /**
     * The absolute path to the storage location of
     * the website's config files  (/config).
     * @var string
     */
    protected $websiteConfigPath = null;

    /**
     * The absolute path to the website's data repository.
     * Formerly known as ASCMS_INSTANCE_PATH.
     * @var string
     * @access protected
     */
    protected $websitePath = null;

    /**
     * The offset path from the website's data repository to the
     * location of the Contrexx installation if it is run in a subdirectory.
     * Formerly known as ASCMS_INSTANCE_OFFSET.
     * @var string
     * @access protected
     */
    protected $websiteOffsetPath = null;

    /**
     * The absolute path to the core components (/core)
     * of the Code Base of the Cloudrexx installation.
     * Formerly known as ASCMS_CORE_PATH.
     * @var string
     */
    protected $codeBaseCorePath = null;

    /**
     * The absolute path of the lib folder
     * of the Code Base of the Cloudrexx installation
     * Formerly known as ASCMS_LIBRARY_PATH
     * @var string
     */
    protected $codeBaseLibraryPath  = null;

    /**
     * System mode
     * @var string Mode as string
     * @access protected
     */
    protected $mode = 'minimal';

    /**
     * Initializes the InstallerCx class
     * This includes all class variables which are needed for the installer.
     * @param string $basePath webserver's DocumentRoot path - if it includes /installer it will be cutted out
     * @access public
     */
    public function __construct($basePath) {
        $this->basePath = str_replace('/installer', '', $basePath);
        $this->codeBaseDocumentRootPath = $this->basePath;
        $this->websiteDocumentRootPath = $this->basePath;
        $this->websiteConfigPath = $this->websiteDocumentRootPath . self::FOLDER_NAME_CONFIG;
        $this->websitePath = $this->basePath;
        $this->websiteOffsetPath = '';
        $this->codeBaseCorePath = $this->codeBaseDocumentRootPath . self::FOLDER_NAME_CORE;
        $this->codeBaseLibraryPath = $this->codeBaseDocumentRootPath . self::FOLDER_NAME_LIBRARY;
    }

    /**
     * Return the absolute path to the Code Base of the Contrexx installation.
     * Formerly known as ASCMS_DOCUMENT_ROOT.
     * @return string
     * @access public
     */
    public function getCodeBaseDocumentRootPath() {
        return $this->codeBaseDocumentRootPath;
    }

    /**
     * Return the absolute path to the data repository of the Contrexx installation.
     * Formerly known as ASCMS_INSTANCE_DOCUMENT_ROOT.
     * @return string
     * @access public
     */
    public function getWebsiteDocumentRootPath() {
        return $this->websiteDocumentRootPath;
    }

    /**
     * Return the absolute path to the storage location of the website's config files.
     * @return string
     */
    public function getWebsiteConfigPath() {
        return $this->websiteConfigPath;
    }

    /**
     * Return the offset path from the website's data repository to the
     * location of the Contrexx installation if it is run in a subdirectory.
     * Formerly known as ASCMS_INSTANCE_OFFSET.
     * @return string
     * @access public
     */
    public function getWebsiteOffsetPath()
    {
        return $this->websiteOffsetPath;
    }

    /**
     * Return the absolute path to the core components (/core)
     * of the Code Base of the Cloudrexx installation.
     * Formerly known as ASCMS_CORE_PATH.
     * @return string
     */
    public function getCodeBaseCorePath() {
        return $this->codeBaseCorePath;
    }

    /**
     * Return the absolute path to the website's data repository.
     * Formerly known as ASCMS_INSTANCE_PATH.
     * @return string
     * @access public
     */
    public function getWebsitePath() {
        return $this->websitePath;
    }

    /**
     * The absolute path of the lib folder
     * of the Code Base of the Cloudrexx installation
     * Formerly known as ASCMS_LIBRARY_PATH
     * @return string
     */
    public function getCodeBaseLibraryPath() {
        return $this->codeBaseLibraryPath;
    }

    /**
     * Returns the mode this instance of Cx is in
     * @return string at the moment it always returns the standard value
     * @access public
     */
    public function getMode() {
        return $this->mode;
    }

    public function setId($id) {
        $this->id = $id;
    }

    public function getId() {
        return $this->id;
    }

    public function getEvents() {
        return new class {public function triggerEvent($eventName, $eventArgs = array()){}};
    }

    public function getDb() {
        if (isset($this->db)) {
            return $this->db;
        }

        $_DBCONFIG['host'] = $_SESSION['installer']['config']['dbHostname'];
        $_DBCONFIG['database'] = $_SESSION['installer']['config']['dbDatabaseName'];
        $_DBCONFIG['tablePrefix'] = $_SESSION['installer']['config']['dbTablePrefix'];
        $_DBCONFIG['user'] = $_SESSION['installer']['config']['dbUsername'];
        $_DBCONFIG['password'] = $_SESSION['installer']['config']['dbPassword'];
        $_DBCONFIG['dbType'] = 'mysql';    // Database type (e.g. mysql,postgres ..)
        $_DBCONFIG['charset'] = 'utf8'; // Charset (default, latin1, utf8, ..)
        $_DBCONFIG['timezone'] = 'UTC'; // Controller's timezone for model
        $_DBCONFIG['collation'] = $_SESSION['installer']['config']['dbCollation'];

        // Set database connection details
        $objDb = new \Cx\Core\Model\Model\Entity\Db($_DBCONFIG);

        // Set database user details
        $objDbUser = new \Cx\Core\Model\Model\Entity\DbUser($_DBCONFIG);

        // Initialize database connection
        $this->db = new \Cx\Core\Model\Db($objDb, $objDbUser, new \Doctrine\Common\Cache\ArrayCache());

        return $this->db;
    }

    public function getComponent($name) {
        if ($name == 'Cache') {
            return new class {
                public function clearCache(){}
                public function delete(){}
                public function fetch(){}
                public function save(){}
            };
        }
    }

    public function getLicense() {
            return new class {
                public function getLegalComponentsList(){
                    return array('SystemInfo');
                }
            };
    }
}
